<?php

class ModelExtensionModuleCart2cartCustomer extends Model
{
  const HASH_METHOD_MD5 = 'md5';
  const HASH_METHOD_SHA1 = 'sha1';
  const HASH_METHOD_SHA256 = 'sha256';
  const HASH_METHOD_MIXED = 'mixed';

  protected $_db;
  protected $_validators;
  protected $_customerInfo;

  public function __construct($registry)
  {
    parent::__construct($registry);

    $this->_db = $registry->get('db');
  }

  public function setCustomerInfo(array $customerInfo)
  {
    $this->_customerInfo = $customerInfo;

    return $this;
  }

  public function login($email, $password)
  {
    $used = [];
    if (!$passworddata = $this->_getPasswordData()) {
      return false;
    }

    foreach ($this->_getValidators() as $cartId => $methods) {
      foreach ($methods as $type => $method) {
        if (!isset($used[$type])) {
          if ($method($passworddata['password_hash'], $password, $passworddata['password_salt'])) {
            $this->_updateCustomerPassword($password);
            $this->_deletePasswordData();
            return true;
          }

          $type != self::HASH_METHOD_MIXED and $used[$type] = true;
        }
      }
    }

    return false;
  }

  protected function _getValidators()
  {
    if ($this->_validators === null) {
      $this->_validators = [
        'Cscart' => [
          self::HASH_METHOD_MD5 => function ($hash, $password, $salt) {
            return (md5($password) == $hash);
          },
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {
            return md5(md5($password) . md5($salt)) == $hash;
          }
        ],
        'Magento' => [
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {

            $argonPasswordsArr = $this->model_extension_module_cart2cart_passwordHash->getArgonPasswordMage(
              $password, $salt, $hash
            );

            if ($possibleSalt = explode(':', $hash)[1] ?? '') {
              $argonPasswordsArr = array_merge(
                $argonPasswordsArr, $this->model_extension_module_cart2cart_passwordHash->getArgonPasswordMage(
                $password, $possibleSalt, $hash
              )
              );
            }
            
            return (md5($salt . $password) === $hash
              || md5($salt . $password) . ':' . $salt === $hash
              || hash('sha256', $salt . $password) === $hash
              || hash('sha256', $salt . $password) . ':' . $salt === $hash
              || in_array($hash, $argonPasswordsArr, true)
              || in_array($hash . ':' . $salt, $argonPasswordsArr, true)
              || ($possibleSalt && in_array($hash . ':' . $possibleSalt, $argonPasswordsArr, true))
            );
          }
        ],
        'Opencart' => [
          self::HASH_METHOD_MD5 => function ($hash, $password, $salt) {
            return (md5($password) == $hash);
          },
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {
            return sha1($salt . sha1($salt . sha1($password))) == $hash;
          },
          self::HASH_METHOD_SHA1 => function ($hash, $password, $salt) {
            return sha1($salt . sha1($salt . sha1($password))) . ':' . $salt == $hash;
          }
        ],
        'Oscommerce' => [
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {
            return $this->model_extension_module_cart2cart_passwordHash->setData(10, true)->checkPassword($password, $hash);
          }
        ],
        'PrestaShop' => [
          self::HASH_METHOD_MD5 => function ($hash, $password, $salt) {
            return (md5($password) == $hash);
          },
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {
            if ((md5($salt . $password) == $hash)) {
              return true;
            }

            return function_exists('password_verify') && password_verify($password, $hash);
          }
        ],
        'Woocommerce' => [
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {
            if (strlen($hash) <= 32) {
              return $this->_hashEquals($hash, md5($password));
            }

            return $this->model_extension_module_cart2cart_passwordHash->setData(8, true)->checkPassword($password, $hash);
          }
        ],
        'Virtuemart' => [
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {
            $encrypted = ($salt) ? md5($password . $salt) : md5($password);
            if ($hash == $encrypted) {
              return true;
            }

            if (strpos($hash, '$P$') === 0) {
              return $this->model_extension_module_cart2cart_passwordHash->setData(10, true)->checkPassword($password, $hash);
            } elseif ($hash[0] == '$') {
              return $this->_passwordVerify($password, $hash);
            } elseif (substr($hash, 0, 8) == '{SHA256}') {
              $parts     = explode(':', $hash);
              $salt      = @$parts[1];

              $encrypted = ($salt) ? hash('sha256', $password . $salt) . ':' . $salt : hash('sha256', $password);
              $testcrypt = '{SHA256}' . $encrypted;

              return $this->_timingSafeCompare($hash, $testcrypt);
            }

            $parts = explode(':', $hash);
            $salt  = @$parts[1];

            $testcrypt = md5($password . $salt) . ($salt ? ':' . $salt : (strpos($hash, ':') !== false ? ':' : ''));

            return $this->_timingSafeCompare($hash, $testcrypt);
          }
        ],
        'Zencart' => [
          self::HASH_METHOD_MIXED => function ($hash, $password, $salt) {
            return $this->model_extension_module_cart2cart_zcHasher->bootstrap()->validatePassword($password, $hash, $salt);
          }
        ],
      ];
    }

    return $this->_validators;
  }

  protected function _getPasswordData()
  {
    $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "c2c_data` WHERE `customer_id` = " . (int)$this->_customerInfo['customer_id']);

    return $query->row;
  }

  protected function _updateCustomerPassword($password)
  {
    $this->db->query("
      UPDATE 
        " . DB_PREFIX . "customer 
      SET 
        password = '" . $this->db->escape(md5($password)) . "'
      WHERE 
        customer_id = " . (int)$this->_customerInfo['customer_id']
    );
  }

  protected function _deletePasswordData()
  {
    $this->db->query("DELETE FROM " . DB_PREFIX . "c2c_data WHERE customer_id = " . (int)$this->_customerInfo['customer_id']);
  }

  protected function _hashEquals($a, $b)
  {
    $aLength = strlen($a);
    if ($aLength !== strlen($b)) {
      return false;
    }
    $result = 0;

    for ($i = 0; $i < $aLength; $i++) {
      $result |= ord($a[$i]) ^ ord($b[$i]);
    }

    return $result === 0;
  }

  protected function _passwordVerify($password, $hash)
  {
    if (!function_exists('crypt')) {
      return false;
    }

    $ret = crypt($password, $hash);
    if (!is_string($ret) || $this->_strlen($ret) != $this->_strlen($hash) || $this->_strlen($ret) <= 13) {
      return false;
    }

    $status = 0;
    for ($i = 0; $i < $this->_strlen($ret); $i++) {
      $status |= (ord($ret[$i]) ^ ord($hash[$i]));
    }

    return $status === 0;
  }

  protected function _strlen($string)
  {
    if (function_exists('mb_strlen')) {
      return mb_strlen($string, '8bit');
    }
    return strlen($string);
  }

  protected function _timingSafeCompare($known, $unknown)
  {
    $known .= chr(0);
    $unknown .= chr(0);

    $knownLength = strlen($known);
    $unknownLength = strlen($unknown);

    $result = $knownLength - $unknownLength;

    for ($i = 0; $i < $unknownLength; $i++) {
      $result |= (ord($known[$i % $knownLength]) ^ ord($unknown[$i]));
    }

    return $result === 0;
  }
}