<?php

class ModelExtensionModuleCart2cartPasswordHash extends Model
{
  protected $_itoa64;
  protected $_iterationCountLog2;
  protected $_portableHashes;
  protected $_randomState;

  public function setData($iterationCountLog2, $portableHashes)
  {
    $this->_itoa64 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

    if ($iterationCountLog2 < 4 || $iterationCountLog2 > 31) {
      $iterationCountLog2 = 8;
    }

    $this->_iterationCountLog2 = $iterationCountLog2;

    $this->_portableHashes = $portableHashes;

    $this->_randomState = microtime();
    if (function_exists('getmypid')) {
      $this->_randomState .= getmypid();
    }

    return $this;
  }

  public function getRandomBytes($count)
  {
    $output = '';
    if (is_readable('/dev/urandom') && ($fh = @fopen('/dev/urandom', 'rb'))) {
      $output = fread($fh, $count);
      fclose($fh);
    }

    if (strlen($output) < $count) {
      $output = '';
      for ($i = 0; $i < $count; $i += 16) {
        $this->_randomState =
          md5(microtime() . $this->_randomState);
        $output .=
          pack('H*', md5($this->_randomState));
      }
      $output = substr($output, 0, $count);
    }

    return $output;
  }

  public function encode64($input, $count)
  {
    $output = '';
    $i = 0;
    do {
      $value = ord($input[$i++]);
      $output .= $this->_itoa64[$value & 0x3f];
      if ($i < $count) {
        $value |= ord($input[$i]) << 8;
      }
      $output .= $this->_itoa64[($value >> 6) & 0x3f];
      if ($i++ >= $count) {
        break;
      }
      if ($i < $count) {
        $value |= ord($input[$i]) << 16;
      }
      $output .= $this->_itoa64[($value >> 12) & 0x3f];
      if ($i++ >= $count) {
        break;
      }
      $output .= $this->_itoa64[($value >> 18) & 0x3f];
    } while ($i < $count);

    return $output;
  }

  public function genSaltPrivate($input)
  {
    $output = '$P$';
    $output .= $this->_itoa64[min($this->_iterationCountLog2 + 5, 30)];
    $output .= $this->encode64($input, 6);

    return $output;
  }

  public function cryptPrivate($password, $setting)
  {
    $output = '*0';
    if (substr($setting, 0, 2) == $output) {
      $output = '*1';
    }

    $id = substr($setting, 0, 3);

    if ($id != '$P$' && $id != '$H$') {
      return $output;
    }

    $countLog2 = strpos($this->_itoa64, $setting[3]);
    if ($countLog2 < 7 || $countLog2 > 30) {
      return $output;
    }

    $count = 1 << $countLog2;

    $salt = substr($setting, 4, 8);
    if (strlen($salt) != 8) {
      return $output;
    }

    $hash = md5($salt . $password, true);
    do {
      $hash = md5($hash . $password, true);
    } while (--$count);

    $output = substr($setting, 0, 12);
    $output .= $this->encode64($hash, 16);

    return $output;
  }

  public function genSaltExtended($input)
  {
    $countLog2 = min($this->_iterationCountLog2 + 8, 24);

    $count = (1 << $countLog2) - 1;

    $output = '_';
    $output .= $this->_itoa64[$count & 0x3f];
    $output .= $this->_itoa64[($count >> 6) & 0x3f];
    $output .= $this->_itoa64[($count >> 12) & 0x3f];
    $output .= $this->_itoa64[($count >> 18) & 0x3f];

    $output .= $this->encode64($input, 3);

    return $output;
  }

  public function genSaltBlowfish($input)
  {
    $itoa64 = './ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

    $output = '$2a$';
    $output .= chr(ord('0') + $this->_iterationCountLog2 / 10);
    $output .= chr(ord('0') + $this->_iterationCountLog2 % 10);
    $output .= '$';

    $i = 0;
    do {
      $c1 = ord($input[$i++]);
      $output .= $itoa64[$c1 >> 2];
      $c1 = ($c1 & 0x03) << 4;
      if ($i >= 16) {
        $output .= $itoa64[$c1];
        break;
      }

      $c2 = ord($input[$i++]);
      $c1 |= $c2 >> 4;
      $output .= $itoa64[$c1];
      $c1 = ($c2 & 0x0f) << 2;

      $c2 = ord($input[$i++]);
      $c1 |= $c2 >> 6;
      $output .= $itoa64[$c1];
      $output .= $itoa64[$c2 & 0x3f];
    } while (1);

    return $output;
  }

  public function hashPassword($password)
  {
    $random = '';

    if (CRYPT_BLOWFISH == 1 && !$this->_portableHashes) {
      $random = $this->getRandomBytes(16);
      $hash = crypt($password, $this->genSaltBlowfish($random));
      if (strlen($hash) == 60) {
        return $hash;
      }
    }

    if (CRYPT_EXT_DES == 1 && !$this->_portableHashes) {
      if (strlen($random) < 3) {
        $random = $this->getRandomBytes(3);
      }
      $hash = crypt($password, $this->genSaltExtended($random));
      if (strlen($hash) == 20) {
        return $hash;
      }
    }

    if (strlen($random) < 6) {
      $random = $this->getRandomBytes(6);
    }

    $hash = $this->cryptPrivate($password, $this->genSaltPrivate($random));

    if (strlen($hash) == 34) {
      return $hash;
    }

    return '*';
  }

  public function checkPassword($password, $stored_hash)
  {
    $hash = $this->cryptPrivate($password, $stored_hash);
    if ($hash[0] == '*') {
      $hash = crypt($password, $stored_hash);
    }

    return $hash === $stored_hash;
  }

  public function getArgonPasswordMage($password, $salt, $currentHash)
  {
    $result = [];

    $hash = '';
    $version = explode(':', $currentHash)[2] ?? '';

    $argon2Id13 = 2;
    $argon2Id13Agnostic = 3;
    $seedBytes = 32;
    $opsLimit = 2;
    $memLimit = 67108864;
    $saltBytes = 16;
    $hashAlgo = 2;

    if (version_compare(PHP_VERSION, '7.0.0', '>=') && function_exists('sodium_crypto_pwhash')) {
      defined('SODIUM_CRYPTO_SIGN_SEEDBYTES') && $seedBytes = SODIUM_CRYPTO_SIGN_SEEDBYTES;
      defined('SODIUM_CRYPTO_PWHASH_OPSLIMIT_INTERACTIVE') && $opsLimit = SODIUM_CRYPTO_PWHASH_OPSLIMIT_INTERACTIVE;
      defined('SODIUM_CRYPTO_PWHASH_MEMLIMIT_INTERACTIVE') && $memLimit = SODIUM_CRYPTO_PWHASH_MEMLIMIT_INTERACTIVE;
      defined('SODIUM_CRYPTO_PWHASH_SALTBYTES') && $saltBytes = SODIUM_CRYPTO_PWHASH_SALTBYTES;
      defined('SODIUM_CRYPTO_PWHASH_ALG_ARGON2ID13') && $hashAlgo = SODIUM_CRYPTO_PWHASH_ALG_ARGON2ID13;

      $argon2Id13Data = explode('_', $version);

      switch ($argon2Id13Data[0] ?? '') {
        case $argon2Id13Agnostic:
          $hash = $this->getArgonHashMage(
            $password,
            $argon2Id13Data[1] ?? $seedBytes,
            $argon2Id13Data[2] ?? $opsLimit,
            $argon2Id13Data[3] ?? $memLimit,
            $salt,
            $saltBytes,
            $hashAlgo
          );
          break;
        case $argon2Id13:
        default:
          $hash = $this->getArgonHashMage($password, $seedBytes, $opsLimit, $memLimit, $salt, $saltBytes, $hashAlgo);
          break;
      }

      if ($hash !== '') {
        $result = [
          implode(':', [$hash, $salt, $version]),
          implode(':', [$hash, $salt, $argon2Id13]),
          implode(':', [$hash, $salt, $argon2Id13Agnostic]),
          implode(':', [$hash, $salt, $argon2Id13Agnostic . '_' . $seedBytes . '_' . $opsLimit . '_' . $memLimit]),
          $hash . ':' . $salt
        ];
      }
    }

    return $result;
  }

  public function getArgonHashMage($password, $seedBytes, $opsLimit, $memLimit, $salt, $saltBytes, $hashAlgo) {
    if (empty($salt)) {
      return $password;
    }

    if (strlen($salt) < $saltBytes) {
      $salt = str_pad($salt, $saltBytes, $salt);
    } elseif (strlen($salt) > $saltBytes) {
      $salt = substr($salt, 0, $saltBytes);
    }

    return bin2hex(sodium_crypto_pwhash($seedBytes, $password, $salt, $opsLimit, $memLimit, $hashAlgo));
  }
}